/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.common;

import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;

import kieker.analysis.AnalysisController;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.exception.AnalysisAlreadyInstantiatedException;
import kieker.webgui.common.exception.AnalysisAlreadyStartedException;
import kieker.webgui.common.exception.AnalysisInstantiationException;
import kieker.webgui.common.exception.AnalysisNotInstantiatedException;
import kieker.webgui.common.exception.AnalysisNotStartedException;
import kieker.webgui.common.exception.ProjectAlreadyStartedException;

/**
 * This manager is responsible for the currently used and running instances of {@code AnalysisController}. It supplies methods to check the states of analysis,
 * instantiate, start and to stop them. This is also a singleton instance.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
// TODO Make this class threadsafe!!
public final class ACManager {
	/**
	 * This is the log for errors, exceptions etc.
	 */
	private static final Log LOG = LogFactory.getLog(ACManager.class);
	/**
	 * This is the singleton instance of this class.
	 */
	private static final ACManager INSTANCE = new ACManager();
	/**
	 * This list contains the current analysis controllers and their corresponding threads. Not every project does have a controller, but every project can have
	 * maximal one.
	 */
	private final ConcurrentHashMap<String, Analysis> analyses = new ConcurrentHashMap<String, Analysis>();

	/**
	 * Creates a new instance of this class.
	 */
	private ACManager() {
		// No code necessary.
	}

	/**
	 * Delivers the singleton instance of this class.
	 * 
	 * @return The only instance of {@link FSManager}.
	 */
	public static ACManager getInstance() {
		return ACManager.INSTANCE;
	}

	/**
	 * This method tries to stop a running analysis.
	 * 
	 * @param project
	 *            The project to be stopped.
	 * @throws AnalysisNotStartedException
	 *             If the analysis is not running or has not yet been initialized.
	 * @throws InterruptedException
	 *             If the current thread has somehow been interrupted while waiting for the existing analysis thread.
	 */
	public void stopAnalysisController(final String project) throws AnalysisNotStartedException {
		final Analysis analysis = this.analyses.get(project);

		if (analysis != null) {
			analysis.stop();
		} else {
			throw new AnalysisNotStartedException("The analysis has not been started yet.");
		}
	}

	/**
	 * This method tries to start the analysis for the given project. It has already to be initialized.
	 * 
	 * @param project
	 *            The project to be started.
	 * @throws ProjectAlreadyStartedException
	 *             If the project is already running.
	 * @throws AnalysisNotInstantiatedException
	 *             If the analysis has not yet been initialized.
	 */
	public void startAnalysisController(final String project) throws AnalysisAlreadyStartedException, AnalysisNotInstantiatedException {
		final Analysis analysis = this.analyses.get(project);

		if (analysis != null) {
			analysis.start();
		} else {
			throw new AnalysisNotInstantiatedException("The analysis has not been initialized yet.");
		}
	}

	/**
	 * This method cleans the analysis controller of the given project. In other words: The potential running instance is stopped and removed. After a call to this
	 * method, the project can be restarted. Potential exceptions will be ignored
	 * 
	 * @param project
	 *            The name of the project to be cleaned.
	 */
	public void cleanAnalysisController(final String project) {
		final Analysis analysis = this.analyses.get(project);

		if (analysis != null) {
			analysis.stop();
			this.analyses.remove(project);
		}
	}

	/**
	 * This method tries to instantiate the analysis controller of the given project using the data available on the file system.
	 * 
	 * @param project
	 *            The project whose analysis controller should be instantiated.
	 * @throws AnalysisInstantiationException
	 */
	public void instantiateAnalysisController(final String project) throws AnalysisAlreadyInstantiatedException, AnalysisInstantiationException {
		try {
			if (!this.analyses.containsKey(project)) {
				FSManager.getInstance().openProject(project);
				FSManager.getInstance().getClassLoader(project);

				// TODO
				// this.analyses.put(project, new Analysis(classLoader, modelProject));
			} else {
				throw new AnalysisAlreadyInstantiatedException("The analysis has already been initialized.");
			}
		} catch (final IOException ex) {
			throw new AnalysisInstantiationException("An IO-error occured while instantiating the analysis.", ex);
		}
	}

	/**
	 * This method can be used to deliver the state of the analysis controller of the given project as a human readable string.
	 * 
	 * @param project
	 *            The project whose state should be delivered.
	 * @return The current state of the corresponding {@link AnalysisController} if it exists, '' otherwise.
	 */
	public String getAnalysisControllerStateString(final String project) {
		final Enum<?> state = this.getAnalysisControllerState(project);

		if (state != null) {
			return state.toString();
		} else {
			return null;
		}
	}

	/**
	 * This method can be used to deliver the state of the analysis controller of the given project as a human readable string.
	 * 
	 * @param project
	 *            The project whose state should be delivered.
	 * @return The current state of the corresponding {@link AnalysisController} if it exists, null otherwise.
	 */
	public Enum<?> getAnalysisControllerState(final String project) {
		final Analysis analysis = this.analyses.get(project);

		if (analysis != null) {
			return analysis.getCurrentState();
		} else {
			return null;
		}
	}

	/**
	 * Delivers the display object for the given display in the given view in the given project.
	 * 
	 * @param project
	 *            The name of the project.
	 * @param viewName
	 *            The name of the view.
	 * @param displayName
	 *            The name of the display.
	 * @return The display object if it exists, null otherwise.
	 */
	public Object getDisplay(final String project, final String viewName, final String displayName) {
		final Analysis analysis = this.analyses.get(project);
		if (analysis != null) {
			return analysis.getDisplay(viewName, displayName);
		} else {
			return null;
		}
	}
}
